package suicideChess;

import java.util.ArrayList;
import java.util.Random;

import suicideChess.Board.NoPieceOnSquare;
import suicideChess.Square.NotAValidSquare;

/**
 * This class will contain all the AI.
 * @author Jean-Baptiste H&eacute;tier
 * @version $LastChangedRevision$, $LastChangedDate$
 */

public class ComputerPlayer {



    /**
     * This asks the computer to compute a move
     * @param bitboard The current status of the {@link Board}
     * @param color The color to play
     * @return move A {@link Move}
     * @throws NotAValidSquare 
     * @see Board
     * @see Move
     */
    public static Move doRandomMove(Board bitboard) throws NotAValidSquare {
        Random generator = new Random();
        Rules.legalMovesForPlayer(bitboard);
        ArrayList<Move> allLegalMoves = Rules.getLegalMovesCapture();
        if (allLegalMoves.size()==0) {
            allLegalMoves = Rules.getLegalMovesNonCapture();
        }
        if (allLegalMoves.size()!=0) {
            return allLegalMoves.get(generator.nextInt(allLegalMoves.size()));
        }
        
        throw new RuntimeException("**Error** in doRandomMove");

    }
    
    /**
     * Basic MinMax
     * @param bitboard The bitboard
     * @param color The color to play
     * @return The best Move found 
     * @throws NotAValidSquare 
     * @throws NoPieceOnSquare 
     * @see Board
     * @see Move
     */
    public static Move doMinMaxMove(Board bitboard) throws NotAValidSquare, NoPieceOnSquare {
        bestMove = null;
        nodesSearched = 0;
        int bestScore = MinMax(bitboard, 0);
        if (SuicideChess.postThinkingOutput()) {
            System.out.println(SuicideChess.PLY_DEPTH+" "+bestScore*100+" 0 "+nodesSearched+" "+bestMove);
        }
        return bestMove;
    }
    
    
    private static int MinMax(Board bitboard, int currentDepth) throws NotAValidSquare, NoPieceOnSquare {
        if (currentDepth >= SuicideChess.PLY_DEPTH) {
            return bitboard.getBoardValue();
        }
        
        nodesSearched++;
        
        Rules.legalMovesForPlayer(bitboard);
        ArrayList<Move> allLegalMoves = Rules.getLegalMovesCapture();
        if (allLegalMoves.size()==0) {
            allLegalMoves = Rules.getLegalMovesNonCapture();
        }
        if (allLegalMoves.size()==0) {
            if (bitboard.getCurrentPlayer()==Piece.BLACK) {
                return Board.BLACK_WINS;
            } else {
                return Board.WHITE_WINS;
            }
        } else {
            int bestMoveIndex=0;
            int currentScore;
            int bestScoreSoFar;
            if (bitboard.getCurrentPlayer()==Piece.BLACK) {            
                bestScoreSoFar=Board.WHITE_WINS+1; //any move even a WHITE_WINS will be better than that
                for (int i=0; i<allLegalMoves.size(); i++) {
                    Board boardCopy = new Board(bitboard);
                    boardCopy.doMove(allLegalMoves.get(i));
                    currentScore=MinMax(boardCopy,currentDepth+1);
                    if (currentScore < bestScoreSoFar) { //black tries to minimise his score
                        bestScoreSoFar = currentScore;
                        bestMoveIndex = i;
                    }
                }
            } else { //white piece
                bestScoreSoFar=Board.BLACK_WINS-1; //any move even a BLACK_WINS will be better than that
                for (int i=0; i<allLegalMoves.size(); i++) {
                    Board boardCopy = new Board(bitboard);
                    boardCopy.doMove(allLegalMoves.get(i));
                    currentScore=MinMax(boardCopy,currentDepth+1);
                    if (currentScore > bestScoreSoFar) { //white tries to maximise his score
                        bestScoreSoFar = currentScore;
                        bestMoveIndex = i;
                    }
                }
            }
            bestMove = allLegalMoves.get(bestMoveIndex);
            return bestScoreSoFar;
        }  
    }
    
    private static Move bestMove = null;
    private static int nodesSearched;
}